package
{
	import flash.display.Sprite;
	import flash.events.Event;
	
	import gs.TweenMax;
	
	import hype.framework.behavior.AbstractBehavior;
	import hype.framework.core.TimeType;
	import hype.framework.rhythm.SimpleRhythm;
	
	public class Asset extends Sprite
	{
		public static const DESTROYED:String				= "DESTROYED";
		
		private var __clip:Sprite;																// reference to the sprite being effected
		private var __time:Number;																// timer to dictate the life of this asset
		private var __alpha:Number;																// maximum alpha value for this asset
		private var __behaviors:Vector.<AbstractBehavior>	= new Vector.<AbstractBehavior>();	// all behaviours attached to this asset
		private var __total:uint;																// total number of behaviors attached to this asset
		private var __interval:SimpleRhythm;													// HYPE SimpleRhythm for using to keep track of this assets life span
		
		public function Asset(s:Sprite, t:Number, a:Number, b:Array)
		{
			// set references
			__clip		= s;
			__time		= t;
			__alpha		= a;
			__total		= b.length;
			
			// copy all of the behaviours from the array past
			for (var i:uint=0; i<__total; i++) __behaviors.push(b[i]);
			
			show();
		}
		
		// START ALL OF THE BEHAVIORS
		public function start():void
		{
			__interval	= new SimpleRhythm(hide);						// setup rhythm
			__interval.start(TimeType.TIME, __time);					// start rhythm
			for (var i:uint=0; i<__total; i++) __behaviors[i].start();	// start all behaviors
		}
		
		// STOP ALL OF THE BEHAVIORS
		public function stop():void
		{
			__interval.stop();											// stop rhythm
			for (var i:uint=0; i<__total; i++) __behaviors[i].stop();	// stop all behaviors
		}
		
		// SHOW / HIDE ASSET
		private function show():void
		{
			clip.alpha	= 0;
			TweenMax.to(clip, .3, { alpha: __alpha });
		}
		
		private function hide(rhythm:SimpleRhythm=null):void
		{
			TweenMax.to(clip, .3, { alpha: 0, onComplete: destroy });
		}
		
		// DESTROY ALL OF THE BEHAVIORS
		public function destroy():void
		{
			// remove all behaviors
			for (var i:uint=0; i<__total; i++)
			{
				__behaviors[i].destroy();
				__behaviors[i]	= null;
			}
			
			// stop rhythm
			__interval.stop();
			
			// remove clip from display
			if (__clip.parent != null)	__clip.parent.removeChild(__clip);
			
			// null vars
			__behaviors	= null;
			__clip		= null;
			__time		= 0;
			__total		= 0;
			
			this.dispatchEvent(new Event(Asset.DESTROYED));
		}
		
		// reference to the clip past in
		public function get clip():Sprite		{ return __clip; }
	}
}