package
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.filters.BlurFilter;
	import flash.filters.GlowFilter;
	import flash.geom.Point;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.net.URLRequest;
	import flash.ui.Keyboard;
	
	import hype.extended.behavior.Swarm;
	import hype.extended.behavior.VariableVibration;
	import hype.extended.color.ColorPool;
	import hype.extended.color.PixelColorist;
	import hype.extended.layout.ShapeLayout;
	import hype.extended.rhythm.FilterCanvasRhythm;
	import hype.extended.util.ContextSavePNG;
	import hype.framework.core.ObjectPool;
	import hype.framework.core.TimeType;
	import hype.framework.display.BitmapCanvas;
	import hype.framework.rhythm.AbstractRhythm;
	import hype.framework.rhythm.SimpleRhythm;
	import hype.framework.sound.SoundAnalyzer;
	
	public class MusicDrawing extends Sprite
	{
		private var __w:int									= stage.stageWidth;							// reference to stage width and height
		private var __h:int									= stage.stageHeight;
		private var __canvasScale:int						= 8;										// scale factor for the overall size of the canvas we want to export
		private var __totalAssets:int;																	// total number of assets on screen
		private var __minScale:Number						= .005;										// minimum scale of an asset
		private var __maxScale:Number						= 5;										// maximum scale of an asset
		private var __scale:Number							= 1;										// current scale of an asset
		
		// list of all the linked assets in your library to use in this app
		private var __linkedAssets:Array					= [Oval, Triangle, Cross, Donut, Shapes, Arrow, Square];
		private var __linkedImages:Array					= [CALogo, ClownFish, ColourMess, Rainbow];
		
		private var __img:Class								= __linkedImages[0];						// selected image for colour reference when colouring with the PixelColorist Class
		
		private var __holder:Sprite							= new Sprite();								// Sprite to hold all of the created assets
		private var __layout:ShapeLayout;																// HYPE layout class
		private var __canvas:BitmapCanvas					= new BitmapCanvas(__w, __h);				// HYPE bitmap canvas, similar to a bitmapData object
		private var __pixelColour:PixelColorist;														// HYPE colour class for using linked movies for colour reference
		private var __colourPool:ColorPool					= new ColorPool();							// HYPE colour class for setting up a palette of colours to use at any time
		private var __savePNG:ContextSavePNG;															// HYPE image exporting class, this works with the right mouse click in the Flash context menu
		private var __blurRhythm:FilterCanvasRhythm;													// HYPE filter rhythm, this type of loop must be used when wanting to use filters in conjunction with the SavePNG method
		private var __glowRhythm:FilterCanvasRhythm;													// HYPE filter rhythm, this type of loop must be used when wanting to use filters in conjunction with the SavePNG method
		private var __colourToMovie:SimpleRhythm;														// HYPE simple rhythm loop for colouring assets to the linked image to the library via PixelColorist class
		
		private var __swarmPoint:Point;																	// point used for moving the swarm class
		
		private var __rhythmArray:Vector.<AbstractRhythm>	= new Vector.<AbstractRhythm>();			// Vector holding HYPE AbstractRhythm's, this is start and stop all rhythms easily
		private var __objPoolArray:Vector.<ObjectPool>		= new Vector.<ObjectPool>();				// Vector holding HYPE ObjectPool's for memory management
		private var __assetArray:Vector.<Asset>				= new Vector.<Asset>();						// Vector holding all created assets
		
		private var __song:String							= "song.mp3";								// music to load in
		private var __sound:Sound;																		// sound class for loading sound
		private var __soundChannel:SoundChannel				= new SoundChannel();						// sound channel class for controlling sounds
		private var __soundAnalyser:SoundAnalyzer;														// HYPE SoundAnalyzer class used for reading the sound frequencies
		
		private var __useMouse:Boolean						= false;									// if set to true then the position of the created assets will be around the mouse position
				
		
		// ----------------------------------------------------------------------------
		// CONSTRUCTOR
		public function MusicDrawing()
		{
			Bounds.visible	= false;						// hide the bounds movie that is sat on the stage
			__layout		= new ShapeLayout(Bounds);		// initalise the layout class with the bounds movie
			
			addChild(__canvas);								// add the BitmapCanvas to the display
			addChild(__holder);								// add the Holder Sprite to the display
			
			setup();										// general setup
			setupControls();								// setup user controls
			setupColourPool();								// setup the colour pool colours
			setupSaveCanvas();								// setup methods for exporting out a PNG image
			setupSound();									// setup sound classes
			setupColourFromMovie();							// setup a default linked image to get colour references from
			
			startCapturing();								// start capturing the canvas
		}
		
		
		// ----------------------------------------------------------------------------
		// GENERAL SETUP
		
		private function setup():void
		{
			// setup a simple loop for rendering
			var loop:SimpleRhythm	= new SimpleRhythm(render);
			loop.start(TimeType.ENTER_FRAME);
		}
		
		private function render(rhythm:SimpleRhythm):void
		{
			// pick assets based on the sound levels of certain octaves
//			if (__soundAnalyser.getOctave(0, 0, 1) > .4 || __soundAnalyser.getOctave(7, 0, 1) > .6) createRandomAsset();
			if (__soundAnalyser.getOctave(0, 0, 1) > .35) createRandomAsset(1);	// get a random asset with a varible vibration behvaiour when a high frequency is over 50%
			if (__soundAnalyser.getOctave(7, 0, 1) > .5) createRandomAsset(2);	// get a random asset with a swarm behvaiour when a low frequency is over 35%
			
			// set the maximum scale based on the sound peaks
			__scale		= __maxScale * __soundChannel.leftPeak;
		}
		
		
		
		
		// ----------------------------------------------------------------------------
		// CONTROLS
		
		// setup listeners for the user interacting with the keyboard
		private function setupControls():void
		{
			stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDown, false, 0, true);
			stage.addEventListener(KeyboardEvent.KEY_UP, keyUp, false, 0, true);
		}
		
		// check which key has been pressed
		private function keyDown(e:KeyboardEvent):void
		{
			switch(e.keyCode)
			{
				case Keyboard.ENTER:
					__useMouse = true;
					break;
				case Keyboard.SPACE:
					clearCanvas();
					break;
				case Keyboard.SHIFT:
					startBlurring();
					break;
				case Keyboard.CONTROL:
					startColourFromMovie();
					break;
			}
		}
		
		// check which key has been released
		private function keyUp(e:KeyboardEvent):void
		{
			switch(e.keyCode)
			{
				case Keyboard.ENTER:
					__useMouse = false;
					break;
				case Keyboard.SHIFT:
					stopBlurring();
					break;
				case Keyboard.CONTROL:
					stopColourFromMovie();
					break;
			}
		}
		
		
		
		
		// ----------------------------------------------------------------------------
		// DRAWING METHODS
		
		// add new asset to display and to the main array holder
		private function addAsset(asset:Asset):void
		{
			__assetArray.push(asset);												// add asset to main array holder
			__holder.addChild(asset.clip);											// add asset to display
			
			asset.addEventListener(Asset.DESTROYED, removeAsset, false, 0, true);	// add listener to see when the asset is destroyed
			asset.start();															// start asset
			__totalAssets++;														// increment total assets
		}
		
		// remove asset from asset array
		private function removeAsset(e:Event):void
		{
			var asset:Asset	= Asset(e.currentTarget);			// get a reference to the asset that called the event
			var pos:int		= __assetArray.indexOf(asset);		// get the position of the asset in the array
			if (pos == -1) return;								// get the position is valid
			__totalAssets--;									// decrease the total asset count
			__assetArray.splice(pos, 1);						// remove from array
		}
		
		// get a random asset with a random behavior
		private function createRandomAsset(type:int=-1):void
		{
			// get random linked asset
			var rand:uint	= Math.round(Math.random()*(__linkedAssets.length-1));
			var s:Sprite	= new __linkedAssets[rand]();
			
			// attach random behavior
			rand			= (type != -1) ? type : Math.round(Math.random()*1);
			if (rand == 1)	createVariableVibration(s);
			else			createSwarm(s);
			
			// get reandom colour process to start with
			rand			= Math.round(Math.random()*1);
			if (rand == 1)	colourFromMovie(s);
			else			colourFromPool(s);
		}
		
		// variable vibration object
		private function createVariableVibration(s:Sprite):void
		{
			// get random values for spring, ease and range values
			var spring:Number		= Math.random()*.7 + .2;
			var ease:Number			= Math.random()*.4 + .1;
			
			var moveRange:Number	= Math.random()* (40*__soundAnalyser.getOctave(1, 0, 1)) + 10;
			var scaleRange			= __soundAnalyser.getOctave(4, __minScale, __scale);
			var rotRange			= __soundAnalyser.getOctave(3, 10, 360);
			
			// get a random position to start at
			var point:Point			= getRandomSoundPoint();
			s.x						= point.x;
			s.y						= point.y;
			
			// setup variable vibration behavior
			var xVib:VariableVibration		= new VariableVibration(s, "x", spring, ease, moveRange);
			var yVib:VariableVibration		= new VariableVibration(s, "y", spring, ease, moveRange);
			var rVib:VariableVibration		= new VariableVibration(s, "rotation", spring, ease, rotRange);
			var sxVib:VariableVibration		= new VariableVibration(s, "scaleX", spring, ease, scaleRange);
			var syVib:VariableVibration		= new VariableVibration(s, "scaleY", spring, ease, scaleRange);
			
			var asset:Asset					= new Asset(s, getLifeSpan(), getAlpha(), [xVib, yVib, rVib, sxVib, syVib]);	// create a new asset and pass in values
			addAsset(asset);				// add to main array holder and also add to display
		}
		
		// swarm object
		private function createSwarm(s:Sprite):void
		{
			// get random values for spring, ease and range values
			var ease:Number			= Math.random()*.1 + .005;
			var speed:Number		= Math.random()* (5*__soundAnalyser.getOctave(1, 0, 1)) + 1;
			var twitch:Number		= __soundAnalyser.getOctave(3, 5, 40);
			var scale				= __soundAnalyser.getOctave(4, __minScale, __scale);
			
			// get a random position to start at and a random position to aim to
			var startPoint:Point	= getRandomSoundPoint();
			var endPoint:Point		= __layout.getNextPoint();
			
			// position and set z depth
			s.x			= startPoint.x;
			s.y			= startPoint.y;
			s.scaleX	= scale;
			s.scaleY	= scale;
			
			// setup variable vibration behavior
			var swarm:Swarm			= new Swarm(s, endPoint, speed, ease, twitch);
			
			var asset:Asset			= new Asset(s, getLifeSpan(), getAlpha(), [swarm]);	// create a new asset and pass in values
			addAsset(asset);															// add to main array holder and also add to display
		}
		
		
		
		
		// ----------------------------------------------------------------------------
		// FILTER METHODS
		
		// BLUR
		// start blurring the canvas
		private function startBlurring():void
		{
			// if the blur filter is not running then lets create it first
			if (__blurRhythm == null)
			{
				__blurRhythm		= new FilterCanvasRhythm([new BlurFilter(1.5, 1.5, 1)], __canvas);	// create new instance of the filter canvas rhythm and create a Blur filter on it
				__blurRhythm.start(TimeType.TIME, 100);													// run this rhythm every 100 milliseconds
				__rhythmArray.push(__blurRhythm);														// add to the main array holder
			}
		}
		
		// stop blurring and remove from main array holder
		private function stopBlurring():void
		{
			stopLoop(__blurRhythm);
			__blurRhythm = null;
		}
		
		/*
		// GLOW
		// start blurring the canvas
		private function startGlowing():void
		{
			// if the blur filter is not running then lets create it first
			if (__glowRhythm == null)
			{
				__glowRhythm		= new FilterCanvasRhythm([new GlowFilter(0x000000, .2)], __canvas);	// create new instance of the filter canvas rhythm and create a Blur filter on it
				__glowRhythm.start(TimeType.TIME, 100);			// run this rhythm every 100 milliseconds
				__rhythmArray.push(__glowRhythm);				// add to the main array holder
			}
		}
		
		// stop blurring and remove from main array holder
		private function stopGlowing():void
		{
			stopLoop(__glowRhythm);
			__glowRhythm = null;
		}
		*/
		
		
		
		// ----------------------------------------------------------------------------
		// COLOUR METHODS
		
		// COLOUR POOL
		// setup the colour pool classes
		private function setupColourPool():void
		{
			// add colours to the colour pool
			// you can add a second argument which specifies the amount of times
			// that a colour is added to the pool to increase its chances of being picked at random
			__colourPool.addColor(0xFFFFFF, 3);
			__colourPool.addColor(0xF7F7F7, 3);
			__colourPool.addColor(0xECECEC, 3);
			__colourPool.addColor(0xCCCCCC, 3);
			__colourPool.addColor(0x999999, 3);
			__colourPool.addColor(0x666666, 3);
			__colourPool.addColor(0xFF3300);
			__colourPool.addColor(0xFF6600);
			__colourPool.addColor(0xE55F0A);
			__colourPool.addColor(0xC4520A);
			__colourPool.addColor(0x853711);
			__colourPool.addColor(0x422811);
			__colourPool.addColor(0xF7EFD2);
			__colourPool.addColor(0xF4DF9F);
			__colourPool.addColor(0xF6D16A);
			__colourPool.addColor(0xDDCB3A);
			__colourPool.addColor(0xBDBC35);
			__colourPool.addColor(0x9E9F2B);
			__colourPool.addColor(0x6F7B1D);
			__colourPool.addColor(0xF67605);
			__colourPool.addColor(0xF09724);
		}
		
		// colour the passed sprite a random colour from the colour pool
		private function colourFromPool(s:Sprite):void		{ __colourPool.colorChildren(s); }
		
		
		// COLOUR FROM LINKED MOVIE
		// colour the passed sprite the same colour as its pixel position based on a specified movie in the library
		private function colourFromMovie(s:Sprite):void		{ __pixelColour.colorChildren(s); }
		
		// loop through all assets and colour them based on there position in relation to the linked movie asset
		private function colorAllFromMovie(rhythm:SimpleRhythm=null):void
		{
			for (var i:uint=0; i<__totalAssets; i++) colourFromMovie(__assetArray[i].clip);
		}
		
		// setup the PixelColorist class with the currently selected linked image asset
		private function setupColourFromMovie():void
		{
			__pixelColour	= null;
			__pixelColour	= new PixelColorist(__img, __w, __h);
		}
		
		// start colouring assets to the linked movie
		private function startColourFromMovie():void
		{
			// check to see if we are not already colouring from a movie first
			if (__colourToMovie == null)
			{
				// pick a random image from our image asset array to use for colour reference
				__img			= __linkedImages[Math.round(Math.random()*(__linkedImages.length-1))];
				
				setupColourFromMovie();	// setup the new linked image we have picked to use as a colour reference
				
				__colourToMovie		= new SimpleRhythm(colorAllFromMovie);	// create new simple rhythm for colouring the assets
				__colourToMovie.start(TimeType.TIME, 5);					// run this rhythm every 5 milliseconds
				__rhythmArray.push(__colourToMovie);						// add to the main array holder
			}
		}
		
		// start colouring assets to the linked movie and remove from main array holder
		private function stopColourFromMovie():void
		{
			stopLoop(__colourToMovie);
			__colourToMovie = null;
		}
		
		
		
		
		// ----------------------------------------------------------------------------
		// SETUP CANVAS
		
		// start capturing the graphical information into the BitmapCanvas
		private function startCapturing():void
		{
			if (!__canvas.isCapturing) __canvas.startCapture(__holder, true);
		}
		
		// stop capturing the graphical information into the BitmapCanvas
		private function stoptCapturing():void
		{
			if (__canvas.isCapturing) __canvas.stopCapture();
		}
		
		// SETUP EXPORT SETTINGS
		private function setupSaveCanvas():void
		{
			// setupLargeCanvas(10) = scaleFactor x 10 (640 x 360 Stage / 6400 x 3600 PNG)
			// the higher the scale factor to the image you want rendering out for the final PNG
			// the slower the final SWF will run as it will have to handle the calculations
			// exponentially based on the scale factor
			__canvas.setupLargeCanvas(__canvasScale);
			
			__savePNG	= new ContextSavePNG(__canvas);
			__savePNG.onEncodeStart = stopEverything;
		}
		
		// clear canvas contents
		private function clearCanvas():void
		{
			__canvas.clear();
		}
		
		
		
		
		// ----------------------------------------------------------------------------
		// SOUND
		
		// SETUP PLAYING MUSIC AND ANALYSING SOUND
		private function setupSound():void
		{
			__sound				= new Sound(new URLRequest(__song));
			__soundChannel		= __sound.play();
			
			__soundAnalyser		= new SoundAnalyzer();
			__soundAnalyser.start();
		}
		
		
		
		
		// ----------------------------------------------------------------------------
		// STOP LOOPS
		
		// loop through all of the rhythm loops we have setup and stop them all so that all of our
		// processing power can be used for capturing and exporting our PNG image
		private function stopEverything():void
		{
			for (var i:uint=0; i<__rhythmArray.length; i++)
			{
				var rhythm:AbstractRhythm	= __rhythmArray[i];	// get a reference to the rhythm we are stopping
				rhythm.stop();									// stop the rhythm
			}
			
			for (i=0; i<__assetArray.length; i++)
			{
				var asset:Asset	= __assetArray[i];				// get a reference to the asset we are stopping
				asset.stop();									// stop the asset
			}
			
			__soundChannel.stop();		// stop sound
			__soundAnalyser.stop();		// stop analysing the sound
			stoptCapturing();			// stop capturing
			removeChild(__holder);		// remove holder from the display
		}
		
		// loop through all of the loops contained in the rhythmArray and if we find a match to the passed
		// rhythm then stop the loop, remove it from the array and return out of the loop
		private function stopLoop(rhythm:AbstractRhythm):void
		{
			for (var i:uint=0; i<__rhythmArray.length; i++)
			{
				if (__rhythmArray[i] == rhythm)
				{
					rhythm.stop();
					__rhythmArray.splice(i, 1);
					return;
				}
			}
		}
		
		
		
		
		// ----------------------------------------------------------------------------
		// GET RANDOM VALUES FOR VARIOUS STUFF
		
		// get a random life span for a created asset
		private function getLifeSpan(max:Number=3000):Number	{ return Math.random()*(max*.5)+(max*.5); }
		
		// get a random maximum alpha value for an asset based on the sound
		private function getAlpha(max:Number=1):Number			{ return __soundAnalyser.getOctave(5, .2, max); }
		
		// get a random position based on the sound
		private function getRandomSoundPoint():Point
		{
			// these are a couple of different ways of picking new points to position the assets when they are created
			
			// PICK A NEW POINT BASED AROUND THE CENTER OF THE STAGE AND A RANDOM POINT
//			var point:Point		= __layout.getNextPoint();
//			var x:Number		= __w*.5 + ((point.x - __w*.5) * __soundAnalyser.getOctave(6, 0, 1));
//			var y:Number		= __h*.5 + ((point.y - __h*.5) * __soundAnalyser.getOctave(6, 0, 1));
			
			// CHECK IF WE ARE USING THE MOUSE FOR THE BASE POSITION, IF NOT THEN DO EVERYTHING BASED AROUND THE CENTER OF THE STAGE
			var baseX:Number		= __w * .5;
			var baseY:Number		= __h * .5;
			var moveRangeX:Number	= __w * .5;
			var moveRangeY:Number	= __h * .5;
			
			if (__useMouse)
			{
				baseX		= this.mouseX;
				baseY		= this.mouseY;
				moveRangeX	= 40;
				moveRangeY	= 40;
			}
			
			// PICK A NEW POINT AROUND THE CENTER POINT WITH A SOUND ORIENTED ANGLE TO POSITION IT
			var r:Number		= (360 * __soundAnalyser.getOctave(1, 0, 1)) * 180/Math.PI;
			var x:Number		= baseX + (moveRangeX * __soundAnalyser.getOctave(6, 0, 1)) * Math.cos(r);
			var y:Number		= baseY + (moveRangeY * __soundAnalyser.getOctave(3, 0, 1)) * Math.sin(r);
			
			// return new point
			return new Point(x, y);
		}
	}
}